@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2011 Red Hat, Inc.
+ * Copyright (c) 2011, 2015 Red Hat, Inc.
  *
  * Gnome Documents is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by the
@@ -35,6 +35,7 @@ const Signals = imports.signals;
 const Tweener = imports.tweener.tweener;
 
 const Application = imports.application;
+const ErrorBox = imports.errorBox;
 const MainToolbar = imports.mainToolbar;
 const Places = imports.places;
 const Searchbar = imports.searchbar;
@@ -54,116 +55,131 @@ const PreviewView = new Lang.Class({
         this._controlsFlipId = 0;
         this._controlsVisible = false;
         this._pageChanged = false;
+        this._hasSelection = false;
         this._viewSelectionChanged = false;
         this._fsToolbar = null;
         this._overlay = overlay;
         this._lastSearch = '';
-        this._loadError = false;
 
         Application.modeController.connect('fullscreen-changed', Lang.bind(this,
             this._onFullscreenChanged));
         Application.modeController.connect('window-mode-changed', Lang.bind(this,
             this._onWindowModeChanged));
 
-        this.widget = new Gtk.ScrolledWindow({ hexpand: true,
-                                               vexpand: true });
-        this.widget.get_style_context().add_class('documents-scrolledwin');
-        this.widget.get_hscrollbar().connect('button-press-event', Lang.bind(this, this._onScrollbarClick));
-        this.widget.get_vscrollbar().connect('button-press-event', Lang.bind(this, this._onScrollbarClick));
-        this.widget.get_hadjustment().connect('value-changed', Lang.bind(this, this._onAdjustmentChanged));
-        this.widget.get_vadjustment().connect('value-changed', Lang.bind(this, this._onAdjustmentChanged));
+        this.widget = new Gtk.Stack({ homogeneous: true,
+                                      transition_type: Gtk.StackTransitionType.CROSSFADE });
+
+        this._errorBox = new ErrorBox.ErrorBox();
+        this.widget.add_named(this._errorBox.widget, 'error');
+
+        this._sw = new Gtk.ScrolledWindow({ hexpand: true,
+                                            vexpand: true });
+        this._sw.get_style_context().add_class('documents-scrolledwin');
+        this._sw.get_hscrollbar().connect('button-press-event', Lang.bind(this, this._onScrollbarClick));
+        this._sw.get_vscrollbar().connect('button-press-event', Lang.bind(this, this._onScrollbarClick));
+        this._sw.get_hadjustment().connect('value-changed', Lang.bind(this, this._onAdjustmentChanged));
+        this._sw.get_vadjustment().connect('value-changed', Lang.bind(this, this._onAdjustmentChanged));
+        this.widget.add_named(this._sw, 'view');
 
         this._createView();
 
-        // create page nav bar
-        this._navBar = new PreviewNavBar(this._model);
-        this._overlay.add_overlay(this._navBar.widget);
-
-        // create page nav buttons
-        this._navButtons = new PreviewNavButtons(this, this._overlay);
+        // create context menu
+        let model = this._getPreviewContextMenu();
+        this._previewContextMenu = Gtk.Menu.new_from_model(model);
+        this._previewContextMenu.attach_to_widget(this._sw, null);
 
         this.widget.show_all();
 
-        Application.application.connect('action-state-changed::bookmark-page',
+        this._bookmarkPage = Application.application.lookup_action('bookmark-page');
+        this._bookmarkPage.enabled = false;
+        let bookmarkPageId = this._bookmarkPage.connect('change-state',
             Lang.bind(this, this._onActionStateChanged));
-        this._onActionStateChanged(Application.application, 'bookmark-page',
-            Application.application.get_action_state('bookmark-page'));
+        this._onActionStateChanged(this._bookmarkPage, this._bookmarkPage.state);
 
         this._zoomIn = Application.application.lookup_action('zoom-in');
-        this._zoomIn.connect('activate', Lang.bind(this,
+        let zoomInId = this._zoomIn.connect('activate', Lang.bind(this,
             function() {
                 this._model.set_sizing_mode(EvView.SizingMode.FREE);
                 this.view.zoom_in();
             }));
 
         this._zoomOut = Application.application.lookup_action('zoom-out');
-        this._zoomOut.connect('activate', Lang.bind(this,
+        let zoomOutId = this._zoomOut.connect('activate', Lang.bind(this,
             function() {
                 this._model.set_sizing_mode(EvView.SizingMode.FREE);
                 this.view.zoom_out();
             }));
 
-        this._findPrev = Application.application.lookup_action('find-prev');
-        this._findPrev.connect('activate', Lang.bind(this,
+        let findPrev = Application.application.lookup_action('find-prev');
+        let findPrevId = findPrev.connect('activate', Lang.bind(this,
             function() {
                 this.view.find_previous();
             }));
-        this._findNext = Application.application.lookup_action('find-next');
-        this._findNext.connect('activate', Lang.bind(this,
+        let findNext = Application.application.lookup_action('find-next');
+        let findNextId = findNext.connect('activate', Lang.bind(this,
             function() {
                 this.view.find_next();
             }));
+        this._copy = Application.application.lookup_action('copy');
+        let copyId = this._copy.connect('activate', Lang.bind(this,
+            function() {
+                this.view.copy();
+            }));
 
         let rotLeft = Application.application.lookup_action('rotate-left');
-        rotLeft.connect('activate', Lang.bind(this,
+        let rotLeftId = rotLeft.connect('activate', Lang.bind(this,
             function() {
                 this._changeRotation(-90);
             }));
         let rotRight = Application.application.lookup_action('rotate-right');
-        rotRight.connect('activate', Lang.bind(this,
+        let rotRightId = rotRight.connect('activate', Lang.bind(this,
             function() {
                 this._changeRotation(90);
             }));
         let showPlaces = Application.application.lookup_action('places');
-        showPlaces.connect('activate', Lang.bind(this, this._showPlaces));
+        let showPlacesId = showPlaces.connect('activate', Lang.bind(this, this._showPlaces));
+
+        let nightModeId = Application.application.connect('action-state-changed::night-mode',
+            Lang.bind(this, this._updateNightMode));
 
         this._togglePresentation = Application.application.lookup_action('present-current');
-        Application.application.connect('action-state-changed::present-current',
+        let presentCurrentId = Application.application.connect('action-state-changed::present-current',
             Lang.bind(this, this._onPresentStateChanged));
 
         Application.documentManager.connect('load-started',
                                             Lang.bind(this, this._onLoadStarted));
-        Application.documentManager.connect('load-finished',
-                                            Lang.bind(this, this._onLoadFinished));
         Application.documentManager.connect('load-error',
                                             Lang.bind(this, this._onLoadError));
+
+        this.widget.connect('destroy', Lang.bind(this,
+            function() {
+                this._bookmarkPage.disconnect(bookmarkPageId);
+                this._zoomIn.disconnect(zoomInId);
+                this._zoomOut.disconnect(zoomOutId);
+                findPrev.disconnect(findPrevId);
+                findNext.disconnect(findNextId);
+                this._copy.disconnect(copyId);
+                rotLeft.disconnect(rotLeftId);
+                rotRight.disconnect(rotRightId);
+                showPlaces.disconnect(showPlacesId);
+                Application.application.disconnect(presentCurrentId);
+                Application.application.disconnect(nightModeId);
+            }));
     },
 
     _onLoadStarted: function() {
+        this._bookmarkPage.enabled = false;
         this._showPlaces.enabled = false;
-    },
-
-    _onLoadFinished: function(manager, doc, docModel) {
-        this._showPlaces.enabled = true;
-        this._togglePresentation.enabled = true;
-
-        if (!Application.documentManager.metadata)
-            return;
-
-        this._navButtons.show();
-
-        this._bookmarks = new GdPrivate.Bookmarks({ metadata: Application.documentManager.metadata });
+        this._copy.enabled = false;
     },
 
     _onLoadError: function(manager, doc, message, exception) {
         this._controlsVisible = true;
-
-        this._loadError = true;
         this._syncControlsVisible();
-        this._loadError = false;
+        this._setError(message, exception.message);
     },
 
-    _onActionStateChanged: function(source, actionName, state) {
+    _onActionStateChanged: function(action, state) {
         if (!this._model)
             return;
 
@@ -189,11 +205,19 @@ const PreviewView = new Lang.Class({
     _onPageChanged: function() {
         this._pageChanged = true;
 
+        if (!this._bookmarks)
+            return;
+
         let page_number = this._model.page;
         let bookmark = new GdPrivate.Bookmark({ page_number: page_number });
         let hasBookmark = (this._bookmarks.find_bookmark(bookmark) != null);
 
-        Application.application.change_action_state('bookmark-page', GLib.Variant.new('b', (hasBookmark)));
+        this._bookmarkPage.state = GLib.Variant.new('b', hasBookmark);
+    },
+
+    _setError: function(primary, secondary) {
+        this._errorBox.update(primary, secondary);
+        this.widget.set_visible_child_name('error');
     },
 
     _showPlaces: function() {
@@ -239,8 +263,18 @@ const PreviewView = new Lang.Class({
     },
 
     _onViewSelectionChanged: function() {
+        let hasSelection = this.view.get_has_selection();
+        this._copy.enabled = hasSelection;
+
+        if (!hasSelection &&
+            hasSelection == this._hasSelection) {
+            this._viewSelectionChanged = false;
+            return;
+        }
+
+        this._hasSelection = hasSelection;
         this._viewSelectionChanged = true;
-        if (!this.view.get_has_selection())
+        if (!hasSelection)
             this._cancelControlsFlip();
     },
 
@@ -310,7 +344,7 @@ const PreviewView = new Lang.Class({
 
     _createView: function() {
         this.view = EvView.View.new();
-        this.widget.add(this.view);
+        this._sw.add(this.view);
         this.view.show();
 
         this.view.connect('notify::can-zoom-in', Lang.bind(this,
@@ -321,24 +355,28 @@ const PreviewView = new Lang.Class({
             this._onButtonPressEvent));
         this.view.connect('button-release-event', Lang.bind(this,
             this._onButtonReleaseEvent));
-        this.view.connect('key-press-event', Lang.bind(this,
-            this._onKeyPressEvent));
         this.view.connect('selection-changed', Lang.bind(this,
             this._onViewSelectionChanged));
         this.view.connect('external-link', Lang.bind(this,
             this._handleExternalLink));
+
+        this._navControls = new PreviewNavControls(this, this._overlay);
+        this.widget.set_visible_child_full('view', Gtk.StackTransitionType.NONE);
+    },
+
+    _getPreviewContextMenu: function() {
+        let builder = new Gtk.Builder();
+        builder.add_from_resource('/org/gnome/documents/preview-context-menu.ui');
+        return builder.get_object('preview-context-menu');
     },
 
     _syncControlsVisible: function() {
         if (this._controlsVisible) {
             if (this._fsToolbar)
                 this._fsToolbar.show();
-            if (!this._loadError)
-                this._navBar.show();
         } else {
             if (this._fsToolbar)
                 this._fsToolbar.hide();
-            this._navBar.hide();
         }
     },
 
@@ -347,7 +385,7 @@ const PreviewView = new Lang.Class({
         if (windowMode != WindowMode.WindowMode.PREVIEW) {
             this.controlsVisible = false;
             this._hidePresentation();
-            this._navButtons.hide();
+            this._navControls.hide();
         }
     },
 
@@ -372,36 +410,6 @@ const PreviewView = new Lang.Class({
         this._syncControlsVisible();
     },
 
-    _onKeyPressEvent: function(widget, event) {
-        let keyval = event.get_keyval()[1];
-        let state = event.get_state()[1];
-
-        if ((keyval == Gdk.KEY_Page_Up) &&
-            ((state & Gdk.ModifierType.CONTROL_MASK) != 0)) {
-            this.view.previous_page();
-            return true;
-        }
-
-        if ((keyval == Gdk.KEY_Page_Down) &&
-            ((state & Gdk.ModifierType.CONTROL_MASK) != 0)) {
-            this.view.next_page();
-            return true;
-        }
-
-        if (keyval == Gdk.KEY_Page_Up) {
-            this.view.scroll(Gtk.ScrollType.PAGE_BACKWARD, false);
-            return true;
-        }
-
-        if (keyval == Gdk.KEY_space ||
-            keyval == Gdk.KEY_Page_Down) {
-            this.view.scroll(Gtk.ScrollType.PAGE_FORWARD, false);
-            return true;
-        }
-
-        return false;
-     },
-
     _flipControlsTimeout: function() {
         this._controlsFlipId = 0;
         let visible = this.controlsVisible;
@@ -428,9 +436,15 @@ const PreviewView = new Lang.Class({
      },
 
     _onButtonPressEvent: function(widget, event) {
+        let button = event.get_button()[1];
+
+        if (button == 3) {
+            let time = event.get_time();
+            this._previewContextMenu.popup(null, null, null, button, time);
+            return true;
+        }
 
         this._viewSelectionChanged = false;
-
         return false;
    },
 
@@ -481,6 +495,10 @@ const PreviewView = new Lang.Class({
         this._syncControlsVisible();
     },
 
+    activateResult: function() {
+        this.view.find_next();
+    },
+
     startSearch: function(str) {
         if (!this._model)
             return;
@@ -513,6 +531,13 @@ const PreviewView = new Lang.Class({
         this.emit('search-changed', job.has_results());
     },
 
+    reset: function() {
+        this.setModel(null);
+        this.view.destroy();
+        this._navControls.destroy();
+        this._createView();
+    },
+
     setModel: function(model) {
         if (this._model == model)
             return;
@@ -526,9 +551,29 @@ const PreviewView = new Lang.Class({
 
         if (this._model) {
             this.view.set_model(this._model);
-            this._navBar.setModel(model);
-            this._navButtons.setModel(model);
+            this._navControls.setModel(model);
+            this._navControls.show();
+            this._togglePresentation.enabled = true;
+
+            if (Application.documentManager.metadata)
+                this._bookmarks = new GdPrivate.Bookmarks({ metadata: Application.documentManager.metadata });
+
+            let hasMultiplePages = (this._model.document.get_n_pages() > 1);
+            this._bookmarkPage.enabled = hasMultiplePages && this._bookmarks;
+            this._showPlaces.enabled = hasMultiplePages;
+
             this._model.connect('page-changed', Lang.bind(this, this._onPageChanged));
+
+            this._updateNightMode();
+
+            this.widget.set_visible_child_full('view', Gtk.StackTransitionType.NONE);
+        }
+    },
+
+    _updateNightMode: function() {
+        if (this._model) {
+            let nightMode = Application.settings.get_boolean('night-mode');
+            this._model.set_inverted_colors(nightMode);
         }
     },
 
@@ -547,69 +592,10 @@ const PreviewView = new Lang.Class({
 Signals.addSignalMethods(PreviewView.prototype);
 
 const _PREVIEW_NAVBAR_MARGIN = 30;
-
-const PreviewNavBar = new Lang.Class({
-    Name: 'PreviewNavBar',
-
-    _init: function(model) {
-        this._model = model;
-        this.widget = new GdPrivate.NavBar({ document_model: model,
-                                             margin: _PREVIEW_NAVBAR_MARGIN,
-                                             valign: Gtk.Align.END,
-                                             opacity: 0 });
-        this.widget.get_style_context().add_class('osd');
-
-        let button = new Gtk.Button({ action_name: 'app.places',
-                                      child: new Gtk.Image({ icon_name: 'view-list-symbolic',
-                                                             pixel_size: 16 }),
-                                      valign: Gtk.Align.CENTER
-                                    });
-        let buttonArea = this.widget.get_button_area();
-        buttonArea.pack_start(button, false, false, 0);
-
-        button = new Gtk.ToggleButton({ action_name: 'app.bookmark-page',
-                                        child: new Gtk.Image({ icon_name: 'bookmark-add-symbolic',
-                                                               pixel_size: 16 }),
-                                        valign: Gtk.Align.CENTER
-                                      });
-        buttonArea.pack_start(button, false, false, 0);
-    },
-
-    setModel: function(model) {
-        this._model = model;
-        this.widget.document_model = model;
-        if (!model)
-            this.hide();
-
-        let hasMultiplePages = (model.document.get_n_pages() > 1);
-        Application.application.lookup_action('bookmark-page').enabled = hasMultiplePages;
-        Application.application.lookup_action('places').enabled = hasMultiplePages;
-    },
-
-    show: function() {
-        if (!this._model)
-            return;
-
-        this.widget.show_all();
-        Tweener.addTween(this.widget, { opacity: 1,
-                                        time: 0.30,
-                                        transition: 'easeOutQuad' });
-    },
-
-    hide: function() {
-        Tweener.addTween(this.widget, { opacity: 0,
-                                        time: 0.30,
-                                        transition: 'easeOutQuad',
-                                        onComplete: function() {
-                                            this.widget.hide();
-                                        },
-                                        onCompleteScope: this });
-    }
-});
-
 const _AUTO_HIDE_TIMEOUT = 2;
-const PreviewNavButtons = new Lang.Class({
-    Name: 'PreviewNavButtons',
+
+const PreviewNavControls = new Lang.Class({
+    Name: 'PreviewNavControls',
 
     _init: function(previewView, overlay) {
         this._previewView = previewView;
@@ -617,19 +603,46 @@ const PreviewNavButtons = new Lang.Class({
         this._overlay = overlay;
 
         this._visible = false;
+        this._visibleInternal = false;
         this._pageChangedId = 0;
         this._autoHideId = 0;
         this._motionId = 0;
-        this._hover = false;
 
-        let isRtl = (this._previewView.widget.get_direction() == Gtk.TextDirection.RTL);
-        let prevIconName = isRtl ? 'go-next-symbolic' : 'go-previous-symbolic';
-        let nextIconName = isRtl ? 'go-previous-symbolic' : 'go-next-symbolic';
+        this.bar_widget = new GdPrivate.NavBar({ document_model: this._model,
+                                                 margin: _PREVIEW_NAVBAR_MARGIN,
+                                                 valign: Gtk.Align.END,
+                                                 opacity: 0 });
+        this.bar_widget.get_style_context().add_class('osd');
+        this._overlay.add_overlay(this.bar_widget);
+        this.bar_widget.connect('notify::hover', Lang.bind(this, function() {
+            if (this.bar_widget.hover)
+                this._onEnterNotify();
+            else
+                this._onLeaveNotify();
+        }));
 
-        this.prev_widget = new Gtk.Button({ child: new Gtk.Image ({ icon_name: prevIconName,
+        let buttonArea = this.bar_widget.get_button_area();
+
+        let button = new Gtk.Button({ action_name: 'app.places',
+                                      image: new Gtk.Image({ icon_name: 'view-list-symbolic',
+                                                             pixel_size: 16 }),
+                                      valign: Gtk.Align.CENTER,
+                                      tooltip_text: _("Bookmarks")
+                                    });
+        buttonArea.pack_start(button, false, false, 0);
+
+        button = new Gtk.ToggleButton({ action_name: 'app.bookmark-page',
+                                        image: new Gtk.Image({ icon_name: 'bookmark-new-symbolic',
+                                                               pixel_size: 16 }),
+                                        valign: Gtk.Align.CENTER,
+                                        tooltip_text: _("Bookmark this page")
+                                      });
+        buttonArea.pack_start(button, false, false, 0);
+
+        this.prev_widget = new Gtk.Button({ image: new Gtk.Image ({ icon_name: 'go-previous-symbolic',
                                                                     pixel_size: 16 }),
-                                            margin_left: _PREVIEW_NAVBAR_MARGIN,
-                                            margin_right: _PREVIEW_NAVBAR_MARGIN,
+                                            margin_start: _PREVIEW_NAVBAR_MARGIN,
+                                            margin_end: _PREVIEW_NAVBAR_MARGIN,
                                             halign: Gtk.Align.START,
                                             valign: Gtk.Align.CENTER });
         this.prev_widget.get_style_context().add_class('osd');
@@ -638,10 +651,10 @@ const PreviewNavButtons = new Lang.Class({
         this.prev_widget.connect('enter-notify-event', Lang.bind(this, this._onEnterNotify));
         this.prev_widget.connect('leave-notify-event', Lang.bind(this, this._onLeaveNotify));
 
-        this.next_widget = new Gtk.Button({ child: new Gtk.Image ({ icon_name: nextIconName,
+        this.next_widget = new Gtk.Button({ image: new Gtk.Image ({ icon_name: 'go-next-symbolic',
                                                                     pixel_size: 16 }),
-                                            margin_left: _PREVIEW_NAVBAR_MARGIN,
-                                            margin_right: _PREVIEW_NAVBAR_MARGIN,
+                                            margin_start: _PREVIEW_NAVBAR_MARGIN,
+                                            margin_end: _PREVIEW_NAVBAR_MARGIN,
                                             halign: Gtk.Align.END,
                                             valign: Gtk.Align.CENTER });
         this.next_widget.get_style_context().add_class('osd');
@@ -652,35 +665,56 @@ const PreviewNavButtons = new Lang.Class({
 
         this._overlay.connect('motion-notify-event', Lang.bind(this, this._onMotion));
 
+        this._tapGesture = new Gtk.GestureMultiPress({ propagation_phase: Gtk.PropagationPhase.CAPTURE,
+                                                       touch_only: true,
+                                                       widget: this._previewView.view });
+        this._tapGesture.connect('released', Lang.bind(this, this._onMultiPressReleased));
+        this._tapGesture.connect('stopped', Lang.bind(this, this._onMultiPressStopped));
     },
 
     _onEnterNotify: function() {
-        this._hover = true;
         this._unqueueAutoHide();
         return false;
     },
 
     _onLeaveNotify: function() {
-        this._hover = false;
         this._queueAutoHide();
         return false;
     },
 
     _motionTimeout: function() {
         this._motionId = 0;
+        this._visibleInternal = true;
         this._updateVisibility();
+        this._queueAutoHide();
         return false;
     },
 
-    _onMotion: function() {
+    _onMotion: function(widget, event) {
         if (this._motionId != 0) {
             return false;
         }
 
+        let device = event.get_source_device();
+        if (device.input_source == Gdk.InputSource.TOUCHSCREEN) {
+            return false;
+        }
+
         this._motionId = Mainloop.idle_add(Lang.bind(this, this._motionTimeout));
         return false;
     },
 
+    _onMultiPressReleased: function() {
+        this._tapGesture.set_state(Gtk.EventSequenceState.CLAIMED);
+        this._visibleInternal = !this._visibleInternal;
+        this._unqueueAutoHide();
+        this._updateVisibility();
+    },
+
+    _onMultiPressStopped: function() {
+        this._tapGesture.set_state(Gtk.EventSequenceState.DENIED);
+    },
+
     _onPrevClicked: function() {
         this._previewView.view.previous_page();
     },
@@ -690,9 +724,9 @@ const PreviewNavButtons = new Lang.Class({
     },
 
     _autoHide: function() {
-        this._fadeOutButton(this.prev_widget);
-        this._fadeOutButton(this.next_widget);
         this._autoHideId = 0;
+        this._visibleInternal = false;
+        this._updateVisibility();
         return false;
     },
 
@@ -710,12 +744,15 @@ const PreviewNavButtons = new Lang.Class({
     },
 
     _updateVisibility: function() {
-        if (!this._model || !this._visible) {
+        if (!this._model || !this._visible || !this._visibleInternal) {
+            this._fadeOutButton(this.bar_widget);
             this._fadeOutButton(this.prev_widget);
             this._fadeOutButton(this.next_widget);
             return;
         }
 
+        this._fadeInButton(this.bar_widget);
+
         if (this._model.page > 0)
             this._fadeInButton(this.prev_widget);
         else
@@ -726,9 +763,6 @@ const PreviewNavButtons = new Lang.Class({
             this._fadeInButton(this.next_widget);
         else
             this._fadeOutButton(this.next_widget);
-
-        if (!this._hover)
-            this._queueAutoHide();
     },
 
     setModel: function(model) {
@@ -738,11 +772,10 @@ const PreviewNavButtons = new Lang.Class({
         }
 
         this._model = model;
+        this.bar_widget.document_model = model;
 
         if (this._model)
             this._pageChangedId = this._model.connect('page-changed', Lang.bind(this, this._updateVisibility));
-
-        this._updateVisibility();
     },
 
     _fadeInButton: function(widget) {
@@ -766,14 +799,22 @@ const PreviewNavButtons = new Lang.Class({
 
     show: function() {
         this._visible = true;
-        this._fadeInButton(this.prev_widget);
-        this._fadeInButton(this.next_widget);
+        this._visibleInternal = true;
+        this._updateVisibility();
+        this._queueAutoHide();
     },
 
     hide: function() {
         this._visible = false;
-        this._fadeOutButton(this.prev_widget);
-        this._fadeOutButton(this.next_widget);
+        this._visibleInternal = false;
+        this._updateVisibility();
+    },
+
+    destroy: function() {
+        this.bar_widget.destroy();
+        this.prev_widget.destroy();
+        this.next_widget.destroy();
+        this._tapGesture = null;
     }
 });
 
@@ -785,54 +826,50 @@ const PreviewToolbar = new Lang.Class({
         this._previewView = previewView;
 
         this.parent();
+        this.toolbar.set_show_close_button(true);
+
+        this._handleEvent = false;
+        this._model = null;
 
         this._searchAction = Application.application.lookup_action('search');
+        this._searchAction.enabled = false;
+
         this._gearMenu = Application.application.lookup_action('gear-menu');
+        this._gearMenu.enabled = false;
 
         // back button, on the left of the toolbar
         let backButton = this.addBackButton();
         backButton.connect('clicked', Lang.bind(this,
             function() {
                 Application.documentManager.setActiveItem(null);
+                Application.modeController.goBack();
                 this._searchAction.enabled = true;
             }));
 
+        // menu button, on the right of the toolbar
+        let previewMenu = this._getPreviewMenu();
+        let menuButton = new Gtk.MenuButton({ image: new Gtk.Image ({ icon_name: 'open-menu-symbolic' }),
+                                              menu_model: previewMenu,
+                                              action_name: 'app.gear-menu' });
+        this.toolbar.pack_end(menuButton);
+
         // search button, on the right of the toolbar
         this.addSearchButton();
 
-        // menu button, on the right of the toolbar
-        let previewMenu = this._getPreviewMenu();
-        let menuButton = new Gd.HeaderMenuButton({ menu_model: previewMenu,
-                                                   action_name: 'app.gear-menu',
-                                                   symbolic_icon_name: 'emblem-system-symbolic' });
-        this.toolbar.pack_end(menuButton);
-
         this._setToolbarTitle();
         this.toolbar.show_all();
 
-        let loadStartId =
-            Application.documentManager.connect('load-started',
-                Lang.bind(this, this._onLoadStarted));
-        let loadFinishId =
-            Application.documentManager.connect('load-finished',
-                Lang.bind(this, this._onLoadFinished));
-
-        this.widget.connect('destroy',
+        this.widget.connect('destroy', Lang.bind(this,
             function() {
-                Application.documentManager.disconnect(loadStartId);
-                Application.documentManager.disconnect(loadFinishId);
-            });
+                this._searchAction.enabled = true;
+            }));
     },
 
-    _onLoadStarted: function() {
-        this._gearMenu.enabled = false;
-        this._searchAction.enabled = false;
-    },
+    _enableSearch: function() {
+        if (!this._model)
+            return;
 
-    _onLoadFinished: function(manager, doc, docModel) {
-        this._gearMenu.enabled = true;
-
-        let evDoc = docModel.get_document();
+        let evDoc = this._model.get_document();
         let hasPages = (evDoc.get_n_pages() > 0);
         let isFind = true;
 
@@ -846,6 +883,7 @@ const PreviewToolbar = new Lang.Class({
         } catch (e) {
         }
 
+        this._handleEvent = (hasPages && isFind);
         this._searchAction.enabled = (hasPages && isFind);
     },
 
@@ -883,6 +921,8 @@ const PreviewToolbar = new Lang.Class({
             return;
 
         this._model = model;
+        this._gearMenu.enabled = true;
+        this._enableSearch();
         this._setToolbarTitle();
     }
 });
@@ -972,7 +1012,7 @@ const PreviewFullscreenToolbar = new Lang.Class({
     _init: function(previewView) {
         this.parent(previewView);
 
-        this.revealer = new Gd.Revealer({ valign: Gtk.Align.START });
+        this.revealer = new Gtk.Revealer({ valign: Gtk.Align.START });
         this.revealer.add(this.widget);
         this.revealer.show();
 
