@@ -38,8 +38,7 @@ const Search = imports.search;
 const TrackerUtils = imports.trackerUtils;
 const Utils = imports.utils;
 
-let collectionManager = null;
-let offsetController = null;
+let documentManager = null;
 let queryBuilder = null;
 let searchCategoryManager = null;
 let searchMatchManager = null;
@@ -48,35 +47,36 @@ let searchController = null;
 let sourceManager = null;
 
 const SEARCH_PROVIDER_IFACE = 'org.gnome.Shell.SearchProvider2';
-const SEARCH_PROVIDER_NAME  = 'org.gnome.Documents.SearchProvider';
 const SEARCH_PROVIDER_PATH  = '/org/gnome/Documents/SearchProvider';
 
 const _SHELL_SEARCH_ICON_SIZE = 128;
 
-const SearchProviderIface = <interface name={SEARCH_PROVIDER_IFACE}>
-<method name="GetInitialResultSet">
-  <arg type="as" direction="in" />
-  <arg type="as" direction="out" />
-</method>
-<method name = "GetSubsearchResultSet">
-  <arg type="as" direction="in" />
-  <arg type="as" direction="in" />
-  <arg type="as" direction="out" />
-</method>
-<method name = "GetResultMetas">
-  <arg type="as" direction="in" />
-  <arg type="aa{sv}" direction="out" />
-</method>
-<method name = "ActivateResult">
-  <arg type="s" direction="in" />
-  <arg type="as" direction="in" />
-  <arg type="u" direction="in" />
-</method>
-<method name = "LaunchSearch">
-  <arg type="as" direction="in" />
-  <arg type="u" direction="in" />
-</method>
-</interface>;
+const SearchProviderIface = '<node> \
+<interface name="org.gnome.Shell.SearchProvider2"> \
+<method name="GetInitialResultSet"> \
+  <arg type="as" direction="in" /> \
+  <arg type="as" direction="out" /> \
+</method> \
+<method name = "GetSubsearchResultSet"> \
+  <arg type="as" direction="in" /> \
+  <arg type="as" direction="in" /> \
+  <arg type="as" direction="out" /> \
+</method> \
+<method name = "GetResultMetas"> \
+  <arg type="as" direction="in" /> \
+  <arg type="aa{sv}" direction="out" /> \
+</method> \
+<method name = "ActivateResult"> \
+  <arg type="s" direction="in" /> \
+  <arg type="as" direction="in" /> \
+  <arg type="u" direction="in" /> \
+</method> \
+<method name = "LaunchSearch"> \
+  <arg type="as" direction="in" /> \
+  <arg type="u" direction="in" /> \
+</method> \
+</interface> \
+</node>';
 
 function _createThumbnailIcon(uri) {
     let file = Gio.file_new_for_uri(uri);
@@ -88,7 +88,7 @@ function _createThumbnailIcon(uri) {
         if (path)
             return new Gio.FileIcon({ file: Gio.file_new_for_path(path) });
     } catch(e) {
-        log(e);
+        log('Unable to create thumbnail icon: ' + e.message);
     }
     return null;
 }
@@ -147,7 +147,7 @@ const CreateCollectionIconJob = new Lang.Class({
                     cursor = object.query_finish(res);
                     cursor.next_async(null, Lang.bind(this, this._onCursorNext));
                 } catch (e) {
-                    log('Error querying tracker: ' + e);
+                    log('Unable to run CreateCollectionIconJob: ' + e.message);
                     this._hasItemIds();
                 }
             }));
@@ -169,7 +169,7 @@ const CreateCollectionIconJob = new Lang.Class({
             try {
                 pixbuf = info.load_icon();
             } catch(e) {
-                log("Unable to load pixbuf: " + e);
+                log("Unable to load pixbuf: " + e.message);
             }
         } else if (icon instanceof Gio.FileIcon) {
             try {
@@ -177,7 +177,7 @@ const CreateCollectionIconJob = new Lang.Class({
                 pixbuf = GdkPixbuf.Pixbuf.new_from_stream(stream,
                                                           null);
             } catch(e) {
-                log("Unable to load pixbuf: " + e);
+                log("Unable to load pixbuf: " + e.message);
             }
         }
 
@@ -191,7 +191,7 @@ const CreateCollectionIconJob = new Lang.Class({
             valid = cursor.next_finish(res);
         } catch (e) {
             cursor.close();
-            log('Error querying tracker: ' + e);
+            log('Unable to read results of CreateCollectionIconJob: ' + e.message);
 
             this._hasItemIds();
         }
@@ -257,7 +257,7 @@ const FetchMetasJob = new Lang.Class({
         job.run(Lang.bind(this,
             function(icon) {
                 if (icon)
-                    meta.pixbuf = icon;
+                    meta.icon = icon;
 
                 this._metas.push(meta);
                 this._jobCollector();
@@ -316,9 +316,9 @@ const FetchIdsJob = new Lang.Class({
     run: function(callback, cancellable) {
         this._callback = callback;
         this._cancellable = cancellable;
-        searchController.setString(this._terms.join(' ').toLowerCase());
+        searchController.setString(this._terms.join(' '));
 
-        let query = queryBuilder.buildGlobalQuery();
+        let query = queryBuilder.buildGlobalQuery(Query.QueryFlags.SEARCH, null);
         Application.connectionQueue.add(query.sparql, this._cancellable, Lang.bind(this,
             function(object, res) {
                 let cursor = null;
@@ -327,7 +327,7 @@ const FetchIdsJob = new Lang.Class({
                     cursor = object.query_finish(res);
                     cursor.next_async(this._cancellable, Lang.bind(this, this._onCursorNext));
                 } catch (e) {
-                    log('Error querying tracker: ' + e);
+                    log('Unable to run FetchIdsJob: ' + e.message);
                     callback(this._ids);
                 }
             }));
@@ -340,7 +340,7 @@ const FetchIdsJob = new Lang.Class({
             valid = cursor.next_finish(res);
         } catch (e) {
             cursor.close();
-            log('Error querying tracker: ' + e);
+            log('Unable to read results of FetchIdsJob: ' + e.message);
 
             this._callback(this._ids);
         }
@@ -359,23 +359,17 @@ const ShellSearchProvider = new Lang.Class({
     Name: 'ShellSearchProvider',
 
     _init: function() {
-        Application.application.hold();
-        Gio.DBus.own_name(Gio.BusType.SESSION,
-                          SEARCH_PROVIDER_NAME,
-                          Gio.BusNameOwnerFlags.NONE,
-                          Lang.bind(this, this._onBusAcquired),
-                          null, null);
-
+        this._impl = Gio.DBusExportedObject.wrapJSObject(SearchProviderIface, this);
         this._cache = {};
         this._cancellable = new Gio.Cancellable();
-
-        Search.initSearch(imports.shellSearchProvider);
     },
 
-    _onBusAcquired: function() {
-        let dbusImpl = Gio.DBusExportedObject.wrapJSObject(SearchProviderIface, this);
-        dbusImpl.export(Gio.DBus.session, SEARCH_PROVIDER_PATH);
-        Application.application.release();
+    export: function(connection) {
+        return this._impl.export(connection, SEARCH_PROVIDER_PATH);
+    },
+
+    unexport: function(connection) {
+        return this._impl.unexport_from_connection(connection);
     },
 
     _returnMetasFromCache: function(ids, invocation) {
@@ -389,14 +383,8 @@ const ShellSearchProvider = new Lang.Class({
             let meta = { id: GLib.Variant.new('s', this._cache[id].id),
                          name: GLib.Variant.new('s', this._cache[id].title) };
 
-            let gicon = this._cache[id].icon;
-            let pixbuf = this._cache[id].pixbuf;
-            let iconstr = gicon ? gicon.to_string() : null;
-            if (iconstr)
-                meta['gicon'] = GLib.Variant.new('s', iconstr);
-            else if (pixbuf)
-                meta['icon-data'] = GdPrivate.create_variant_from_pixbuf(pixbuf);
-
+            let icon = this._cache[id].icon;
+            meta['icon'] = icon.serialize();
             metas.push(meta);
         }
 
